/*
Demo scripts for this blog post:
http://www.brentozar.com/archive/2014/12/careful-adding-indexes-with-alwayson-availability-groups/

Copyright 2014 Brent Ozar Unlimited. End User Licensing Agreement:
http://www.brentozar.com/first-aid/downloads/end-user-license-agreement-eula/
*/


/* On the secondary replica: */
USE AdventureWorks2012;
GO

/* 2. Turn on Query, Include Actual Execution Plan and run this query: */
SELECT Suffix FROM Person.Person;
GO


/* Look at the plan. It's doing a clustered index scan across Person.
Well, let's say we run this query constantly, and we need to add an index on it.
Switch over to the primary and add the index in step 3. */




/* 4. Run the query again: */
SELECT Suffix FROM Person.Person;
GO



/* You get an index scan on Person.IX_Suffix, our newly created index. Awesome.
Let's make things a little more complex - let's add a transaction on the replica:
*/
BEGIN TRAN
SELECT Suffix FROM Person.Person;
GO

/* The query uses the IX_Suffix index, so that's cool. Now switch over to the
primary replica for step 5. */




/* 6. Run the query again while the transaction is still in effect: */
SELECT Suffix FROM Person.Person;
GO



/* BOOM - you'll get an error like this:

Msg 3961, Level 16, State 1, Line 37
Snapshot isolation transaction failed in database 'AdventureWorks2012' because the object accessed by the statement has been modified by a DDL statement in another concurrent transaction since the start of this transaction.  It is disallowed because the metadata is not versioned. A concurrent update to metadata can lead to inconsistency if mixed with snapshot isolation.

So technically, yes, you can drop or disable indexes online in Enterprise,
even with AlwaysOn Availability Groups, but if you've got transactions running
on the secondaries, they can fail.

The index drop is not blocked by the transaction - the index drop blazes right
on through like a freight train. Our transaction has been killed, and you don't
get errors in the log, either. Try rolling it back:
*/

ROLLBACK
GO

/* It's gone. Alright, so let's try again - this time with an index creation.
Start a transaction and issue the select:
*/


BEGIN TRAN
SELECT Suffix FROM Person.Person;
GO


/* Note that in the execution plan, it's doing a clustered index scan.
Now switch over to the primary for step 7. */



/* 8. Run the query again, still with the transaction open. */
SELECT Suffix FROM Person.Person;
GO

/* Kaboom - dead again. The lesson: if we're doing transactions on the
secondary, we even have to be careful with index creations. 

So how about transaction isolation levels and nolock hints? Let's set up for
dirty reads: */

SET TRANSACTION ISOLATION LEVEL READ UNCOMMITTED;
GO
BEGIN TRAN
SELECT Suffix FROM Person.Person WITH (NOLOCK);
GO



/* We get a bunch of different suffixes, and on the execution plan,
we're doing an index scan on IX_Suffix. Now switch over to the primary
for step 9.
*/


/* 10. Run the query again: */
SELECT Suffix FROM Person.Person WITH (NOLOCK);
GO

/* Boom. Even when you play dirty, you still get errors. Bottom line - if you
run transactions on the secondary, you can't go adding indexes willy-nilly. */